@@ -119,7 +119,7 @@ read_pid_stat (pid_t pid, struct pid_stat *st, libcrun_error_t *err)
   if (fd < 0)
     {
       /* The process already exited.  */
-      if (errno == ENOENT)
+      if (errno == ENOENT || errno == ESRCH)
         {
           memset (st, 0, sizeof (*st));
           return 0;
@@ -171,7 +171,7 @@ int
 libcrun_write_container_status (const char *state_root, const char *id, libcrun_container_status_t *status,
                                 libcrun_error_t *err)
 {
-  int ret;
+  int r, ret;
   cleanup_free char *file = get_state_directory_status_file (state_root, id);
   cleanup_free char *file_tmp = NULL;
   size_t len;
@@ -199,46 +199,99 @@ libcrun_write_container_status (const char *state_root, const char *id, libcrun_
   yajl_gen_config (gen, yajl_gen_beautify, 1);
   yajl_gen_config (gen, yajl_gen_validate_utf8, 1);
 
-  yajl_gen_map_open (gen);
-  yajl_gen_string (gen, YAJL_STR ("pid"), strlen ("pid"));
-  yajl_gen_integer (gen, status->pid);
+  r = yajl_gen_map_open (gen);
+  if (UNLIKELY (r != yajl_gen_status_ok))
+    goto yajl_error;
 
-  yajl_gen_string (gen, YAJL_STR ("process-start-time"), strlen ("process-start-time"));
-  yajl_gen_integer (gen, status->process_start_time);
+  r = yajl_gen_string (gen, YAJL_STR ("pid"), strlen ("pid"));
+  if (UNLIKELY (r != yajl_gen_status_ok))
+    goto yajl_error;
+
+  r = yajl_gen_integer (gen, status->pid);
+  if (UNLIKELY (r != yajl_gen_status_ok))
+    goto yajl_error;
+
+  r = yajl_gen_string (gen, YAJL_STR ("process-start-time"), strlen ("process-start-time"));
+  if (UNLIKELY (r != yajl_gen_status_ok))
+    goto yajl_error;
+
+  r = yajl_gen_integer (gen, status->process_start_time);
+  if (UNLIKELY (r != yajl_gen_status_ok))
+    goto yajl_error;
+
+  r = yajl_gen_string (gen, YAJL_STR ("cgroup-path"), strlen ("cgroup-path"));
+  if (UNLIKELY (r != yajl_gen_status_ok))
+    goto yajl_error;
 
-  yajl_gen_string (gen, YAJL_STR ("cgroup-path"), strlen ("cgroup-path"));
   tmp = status->cgroup_path ? status->cgroup_path : "";
-  yajl_gen_string (gen, YAJL_STR (tmp), strlen (tmp));
+  r = yajl_gen_string (gen, YAJL_STR (tmp), strlen (tmp));
+  if (UNLIKELY (r != yajl_gen_status_ok))
+    goto yajl_error;
+
+  r = yajl_gen_string (gen, YAJL_STR ("scope"), strlen ("scope"));
+  if (UNLIKELY (r != yajl_gen_status_ok))
+    goto yajl_error;
 
-  yajl_gen_string (gen, YAJL_STR ("scope"), strlen ("scope"));
   tmp = status->scope ? status->scope : "";
-  yajl_gen_string (gen, YAJL_STR (tmp), strlen (tmp));
+  r = yajl_gen_string (gen, YAJL_STR (tmp), strlen (tmp));
+  if (UNLIKELY (r != yajl_gen_status_ok))
+    goto yajl_error;
 
-  yajl_gen_string (gen, YAJL_STR ("rootfs"), strlen ("rootfs"));
-  yajl_gen_string (gen, YAJL_STR (status->rootfs), strlen (status->rootfs));
+  r = yajl_gen_string (gen, YAJL_STR ("rootfs"), strlen ("rootfs"));
+  if (UNLIKELY (r != yajl_gen_status_ok))
+    goto yajl_error;
 
-  yajl_gen_string (gen, YAJL_STR ("systemd-cgroup"), strlen ("systemd-cgroup"));
-  yajl_gen_bool (gen, status->systemd_cgroup);
+  r = yajl_gen_string (gen, YAJL_STR (status->rootfs), strlen (status->rootfs));
+  if (UNLIKELY (r != yajl_gen_status_ok))
+    goto yajl_error;
 
-  yajl_gen_string (gen, YAJL_STR ("bundle"), strlen ("bundle"));
-  yajl_gen_string (gen, YAJL_STR (status->bundle), strlen (status->bundle));
+  r = yajl_gen_string (gen, YAJL_STR ("systemd-cgroup"), strlen ("systemd-cgroup"));
+  if (UNLIKELY (r != yajl_gen_status_ok))
+    goto yajl_error;
 
-  yajl_gen_string (gen, YAJL_STR ("created"), strlen ("created"));
-  yajl_gen_string (gen, YAJL_STR (status->created), strlen (status->created));
+  r = yajl_gen_bool (gen, status->systemd_cgroup);
+  if (UNLIKELY (r != yajl_gen_status_ok))
+    goto yajl_error;
 
-  yajl_gen_string (gen, YAJL_STR ("detached"), strlen ("detached"));
-  yajl_gen_bool (gen, status->detached);
+  r = yajl_gen_string (gen, YAJL_STR ("bundle"), strlen ("bundle"));
+  if (UNLIKELY (r != yajl_gen_status_ok))
+    goto yajl_error;
 
-  yajl_gen_string (gen, YAJL_STR ("external_descriptors"), strlen ("external_descriptors"));
-  yajl_gen_string (gen, YAJL_STR (status->external_descriptors), strlen (status->external_descriptors));
+  r = yajl_gen_string (gen, YAJL_STR (status->bundle), strlen (status->bundle));
+  if (UNLIKELY (r != yajl_gen_status_ok))
+    goto yajl_error;
 
-  yajl_gen_map_close (gen);
+  r = yajl_gen_string (gen, YAJL_STR ("created"), strlen ("created"));
+  if (UNLIKELY (r != yajl_gen_status_ok))
+    goto yajl_error;
 
-  if (yajl_gen_get_buf (gen, &buf, &len) != yajl_gen_status_ok)
-    {
-      ret = crun_make_error (err, 0, "cannot generate status file");
-      goto exit;
-    }
+  r = yajl_gen_string (gen, YAJL_STR (status->created), strlen (status->created));
+  if (UNLIKELY (r != yajl_gen_status_ok))
+    goto yajl_error;
+
+  r = yajl_gen_string (gen, YAJL_STR ("detached"), strlen ("detached"));
+  if (UNLIKELY (r != yajl_gen_status_ok))
+    goto yajl_error;
+
+  r = yajl_gen_bool (gen, status->detached);
+  if (UNLIKELY (r != yajl_gen_status_ok))
+    goto yajl_error;
+
+  r = yajl_gen_string (gen, YAJL_STR ("external_descriptors"), strlen ("external_descriptors"));
+  if (UNLIKELY (r != yajl_gen_status_ok))
+    goto yajl_error;
+
+  r = yajl_gen_string (gen, YAJL_STR (status->external_descriptors), strlen (status->external_descriptors));
+  if (UNLIKELY (r != yajl_gen_status_ok))
+    goto yajl_error;
+
+  r = yajl_gen_map_close (gen);
+  if (UNLIKELY (r != yajl_gen_status_ok))
+    goto yajl_error;
+
+  r = yajl_gen_get_buf (gen, &buf, &len);
+  if (UNLIKELY (r != yajl_gen_status_ok))
+    goto yajl_error;
 
   if (UNLIKELY (safe_write (fd_write, buf, (ssize_t) len) < 0))
     {
@@ -259,6 +312,12 @@ exit:
     yajl_gen_free (gen);
 
   return ret;
+
+yajl_error:
+  if (gen)
+    yajl_gen_free (gen);
+
+  return yajl_error_to_crun_error (r, err);
 }
 
 int
@@ -456,7 +515,7 @@ libcrun_container_delete_status (const char *state_root, const char *id, libcrun
   if (UNLIKELY (dir == NULL))
     return crun_make_error (err, 0, "cannot get state directory");
 
-  rundir_dfd = open (dir, O_DIRECTORY | O_RDONLY | O_CLOEXEC);
+  rundir_dfd = TEMP_FAILURE_RETRY (open (dir, O_DIRECTORY | O_RDONLY | O_CLOEXEC));
   if (UNLIKELY (rundir_dfd < 0))
     return crun_make_error (err, errno, "cannot open run directory `%s`", dir);
 
@@ -487,16 +546,18 @@ libcrun_free_container_status (libcrun_container_status_t *status)
   free (status->cgroup_path);
   free (status->bundle);
   free (status->rootfs);
+  free (status->external_descriptors);
   free (status->created);
+  free (status->scope);
 }
 
 int
 libcrun_get_containers_list (libcrun_container_list_t **ret, const char *state_root, libcrun_error_t *err)
 {
   struct dirent *next;
-  libcrun_container_list_t *tmp = NULL;
+  cleanup_container_list libcrun_container_list_t *tmp = NULL;
   cleanup_free char *path = get_run_directory (state_root);
-  cleanup_dir DIR *dir;
+  cleanup_dir DIR *dir = NULL;
 
   *ret = NULL;
   dir = opendir (path);
@@ -533,6 +594,7 @@ libcrun_get_containers_list (libcrun_container_list_t **ret, const char *state_r
       tmp = next_container;
     }
   *ret = tmp;
+  tmp = NULL;
   return 0;
 }
 
@@ -594,7 +656,7 @@ int
 libcrun_status_create_exec_fifo (const char *state_root, const char *id, libcrun_error_t *err)
 {
   cleanup_free char *state_dir = libcrun_get_state_directory (state_root, id);
-  cleanup_free char *fifo_path;
+  cleanup_free char *fifo_path = NULL;
   int ret, fd = -1;
 
   ret = append_paths (&fifo_path, err, state_dir, "exec.fifo", NULL);
@@ -616,7 +678,7 @@ int
 libcrun_status_write_exec_fifo (const char *state_root, const char *id, libcrun_error_t *err)
 {
   cleanup_free char *state_dir = libcrun_get_state_directory (state_root, id);
-  cleanup_free char *fifo_path;
+  cleanup_free char *fifo_path = NULL;
   char buffer[1] = {
     0,
   };
@@ -646,7 +708,7 @@ int
 libcrun_status_has_read_exec_fifo (const char *state_root, const char *id, libcrun_error_t *err)
 {
   cleanup_free char *state_dir = libcrun_get_state_directory (state_root, id);
-  cleanup_free char *fifo_path;
+  cleanup_free char *fifo_path = NULL;
   int ret;
 
   ret = append_paths (&fifo_path, err, state_dir, "exec.fifo", NULL);
